require 'mclib'
require 'mclib/event_factory'
require 'logger'

class MCLib::LogParser

  def initialize
    @factory = MCLib::EventFactory.new
  end

  def parse(line)
    meta_data, message = split_data line
    type = type? meta_data
    @factory.build message, meta_data, type
  end

  private

    def type?(meta_data)
      if meta_data.include? 'INFO'
        return :info
      elsif meta_data.include? 'WARN'
        return :warn
      else
        return :unknown
        # fail ArgumentError, "Unable to determine message type #{meta_data}"
      end 
    end

    def split_data(line)
      buffer = []
      occurences = 0
      stopped_at = 0

      line.chars.each_with_index do |char, i|
        if char == ':' && occurences == 2
          # this is the third time we've seen the colon so this is the message delimeter
          stopped_at = i
          break
        elsif char == ':'
          buffer << char
          occurences += 1
        else
          buffer << char
        end 
      end

      stopped_at += 2 # removes trailing colon and space

      return buffer.join, line[stopped_at..-1]
    end
end