require 'mclib'
require 'mclib/log_parser'
require 'eventmapper'
require 'filewatch/tail'

class MCLib::Watcher

  include EventMapper

  attr_accessor :watch_thread

  def initialize(mc_dir = nil)
    if mc_dir.nil?
      mc_dir = MCLib::get_mc_dir
    end
    
    @log_file = mc_dir + '/logs/latest.log'
    @log_watcher = FileWatch::Tail.new
    @parser = MCLib::LogParser.new
  end

  def start
    @watch_thread = Thread.new do
      @log_watcher.tail @log_file
      @log_watcher.subscribe do |path, line|
        if !line.empty? && !line.nil?
          event = @parser.parse line

          # fire the event with the correct event handler api parameters
          delegate_parameters event

          # always fire the :all handlers
          fire :all, event
        end
      end
    end
  end

  def stop
    @watch_thread.kill
  end

  private

    def delegate_parameters(event)
      type = event.class.name.split('::').last.downcase.to_sym
      case type
      when :chat
        fire :chat, event.username, event.text
      when :login
        fire :login, event
      when :warning
        fire :warning, event.text
      end
    end
end